module ProxyAuth
  # Connects to proxy and gets service token
  class Connection
    def service_token
      register_service
      @response && JSON.parse(@response.body)['token']
    end

    private

    def register_service
      @response = http.post('/services', params, headers)
      Rails.logger.info "Service #{service_name} registered."
    rescue Errno::ECONNREFUSED
      Rails.logger.fatal 'Proxy connection refused.'
    end

    def registration_token
      encoder = ActionController::HttpAuthentication::Token
      token = Rails.application.secrets.proxy['token']
      @registration_token ||= encoder.encode_credentials token
    end

    def headers
      {
        'Accept' => 'application/json',
        'Content-Type' => 'application/json',
        'Authorization' => registration_token
      }
    end

    def http
      @http ||= Net::HTTP.new(uri.host, uri.port)
    end

    def params
      {
        service: {
          name: service_name, host: service_url, api_version: service_version
        }
      }.to_json
    end

    def service_name
      @service_name ||= Rails.application.secrets.service['name']
    end

    def service_url
      @service_url ||= Rails.application.secrets.service['url']
    end

    def service_version
      @service_version ||= Rails.application.secrets.service['api_version']
    end

    def uri
      @uri ||= URI.parse(Rails.application.secrets.proxy['url'])
    end
  end
end
